/*
 Rovers Domain
*/

domain {
   types { Rover, Waypoint }

   world {
      predicate CanTraverse(Rover, Waypoint, Waypoint);
      predicate EquipedForSoilAnalysis(Rover);
      predicate EquipedForRockAnalysis(Rover);
      predicate EquipedForImaging(Rover);
      numeric   Distance(Waypoint, Waypoint);
   }

   state {
      function  Waypoint PositionRover(Rover);
      function  Waypoint PositionLander(Lander);
      function  Waypoint PositionSoilSample(Waypoint);
      predicate          HaveSoilAnalysis(Rover, Waypoint);
      predicate          HaveRockAnalysis(Rover, Waypoint);
      predicate          HaveImage(Rover, Waypoint);
      predicate          CommunicatedSoilAnalysis(Waypoint);
      predicate          CommunicatedRockAnalysis(Waypoint);
      predicate          CommunicatedImage(Waypoint);
   }


/************************************
           A c t i o n s 
*************************************/


action Navigage(Rover r, Waypoint orig, Waypoint dest){
   duration:
      normal 1.2 * Distance(orig, dest) Distance(orig, dest)*0.1;
              //constant Distance(orig, dest);
   conditions:
      @start: PosRover(r) = orig;
      @start: CanTraverse(r, orig, dest);
   effects:
      @end:   PosRobot(t) = dest;
}

action SampleSoil(Rover r, Store s, Waypoint p){
    duration:
        uniform 10 20;
    conditions:
       @overall: PosRover(r) = p;
       @start:   EquipedForSoilAnalysis(r);
    effects:
       @end:     HaveSoilAnalysis(r, p);
}

action SampleRock(Rover r, Store s, Waypoint p){
    duration:
        uniform 10 20;
    conditions:
       @overall: PosRover(r) = p;
       @start:   EquipedForRockAnalysis(r);
    effects:
       @end:     HaveRockAnalysis(r, p);
}


action Calibrate(Rover r, Waypoint p){
    duration:
        uniform 10 20;
    conditions:
       @overall: PosRover(r) = p;
       @start:   EquipedForImaging(r);
    effects:
       @end:     Calibrated(r);
}

action TakeImage(Rover r, Waypoint p){
    duration:
       uniform 5 10;
    condition:
       @overall: PosRover(r) = p;
       @start:   Calibrated(r);
    effects:
       @end:     !Calibated(r);
       @end:     HaveImage(r,p);
}

action CommuniateSoilAnalysis(Rover r, Waypoint p){
    duration:
        uniform 10 20;
    condition:
        @start: ChannelFree();
        @start: HavingSoidAnalysis(r,p);
    effects:
        @start: !ChannelFree();
        @end:   ChannelFree();
        @end:   CommunicatedSoilAnalysis(r,p);
}

action CommuniateRockAnalysis(Rover r, Waypoint p){
    duration:
        uniform 10 20;
    condition:
        @start: ChannelFree();
        @start: HavingRockAnalysis(r,p);
    effects:
        @start: !ChannelFree();
        @end:   ChannelFree();
        @end:   CommunicatedRockAnalysis(r,p);
}

action CommuniateImage(Rover r, Waypoint p){
    duration:
        uniform 10 20;
    condition:
        @start: ChannelFree();
        @start: HavingImage(r,p);
    effects:
        @start: !ChannelFree();
        @end:   ChannelFree();
        @end:   CommunicatedImage(r,p);
}

